"use server"
import { Genre, Story, Sub } from "@prisma/client"
import prisma from "./db"
import { revalidatePath } from "next/cache"
import { redirect } from "next/navigation"
import { storySchema, subSchema } from "app/ui/forms/schemas"
import { z } from "zod"


export async function updateField({ datum, table, column, id, pathname }: { datum?: string | number | Genre[], table: string, column: string, id: number, pathname: string }) {
	"use server"
	try {
		const res = await prisma[table].update({
			where: { id },
			data: {
				[column]: datum
			}
		})
		console.log(`updated record in ${table}: ${JSON.stringify(res)}`)
		revalidatePath(pathname)
		return res
	} catch (error) {
		console.error(error)
		return undefined
	}
}

export async function updateGenres({ genres, table, id, pathname }: { genres: { id: number }[], table: string, id: number, pathname: string }) {
	"use server"
	try {
		const res = await prisma[table].update({
			where: { id },
			data: {
				genres: { set: genres }
			}
		})
		console.log(`updated record in ${table}: ${JSON.stringify(res)}`)
		revalidatePath(pathname)
		return res
	} catch (error) {
		console.error(error)
		return undefined
	}
}

export async function updateSub(data: Sub): Promise<Sub | undefined> {
	"use server"
	try {
		subSchema.parse(data)
		const res = await prisma.sub.update({ where: { id: data.id }, data })
		revalidatePath("submission")
		return res
	} catch (error) {
		console.error(error)
		return undefined
	}
}


export async function updateStory(data: Story & { genres: number[] }): Promise<Story | undefined> {
	"use server"
	//prepare data
	const genresArray = data.genres.map((e) => { return { id: e } })
	const storyData = {
		title: data.title,
		word_count: data.word_count,
	}
	//prepare schemas
	const schema = storySchema.omit({ id: true, genres: true })
	const genreSchema = z.object({ id: z.number() })
	const genresSchema = z.array(genreSchema)
	try {
		//validate
		schema.safeParse(storyData)
		genresSchema.safeParse(genresArray)

	} catch (error) {
		console.error(error)
		return undefined
	}
}





